//这是BitSync.v文件的程序清单
module BitSync (
	rst,clk,datain,
	Bit_Sync);
	
	input		rst;   //复位信号，高电平有效
	input		clk;   //FPGA系统时钟:32MHz
	input	 signed [27:0]	datain;    //输入数据
	output Bit_Sync;                //位同步脉冲输出

	
	//双相时钟产生单元：产生周期为码速率的8倍(采样速率)，占空比为1:3的双相时钟信号
	//两路双相时钟相位相差一个clk32时钟周期
	wire clk_d1,clk_d2; 
   clktrans u2(
	    .rst   (rst),
		 .clk32 (clk),
		 .clk_d1(clk_d1),
		 .clk_d2(clk_d2));

	//鉴相单元：对两路积分输出的信号进行鉴相处理
	wire bef_in,aft_in;
	wire signed [31:0] int_i,int_q;	
	phasedetect u3(
	    .rst   (rst),
		 .clk32 (clk),
		 .int_i  (int_i),
		 .int_q  (int_q),
		 .pd_bef (bef_in),
		 .pd_aft (aft_in));

	//随机徘徊滤波器模块
	wire pd_bef,pd_aft;
	completdigfilter u10(
	    .rst   (rst),
		 .clk32 (clk),
		 .bef_in  (bef_in),
		 .aft_in (aft_in),
		 .bef_out (pd_bef),
		 .aft_out (pd_aft));
	
	//单稳态触发器：检测到一个高电平脉冲后，输出4个clk32周期的高电平
	wire pd_before,pd_after;
	monostable u4 (
	    .rst   (rst),
		 .clk32 (clk),
		 .din   (pd_bef),
		 .dout  (pd_before));
		 
	monostable u5 (
	    .rst   (rst),
		 .clk32 (clk),
		 .din   (pd_aft),
		 .dout  (pd_after));									


	//控制分频单元：对两路单稳输出的信号及双相时钟信号进行处理，分频产生相位相差180度的
	//的位同步信号clk_i、clk_q;
	wire clk_i,clk_q;
	controldivfreq u6 (
	    .rst   (rst),
		 .clk32 (clk),
		 .clk_d1 (clk_d1),
		 .clk_d2 (clk_d2),		
		 .pd_before (pd_before) ,
		 .pd_after  (pd_after),
		 .clk_i  (clk_i),
		 .clk_q  (clk_q));

	//位同步形成单元：检测分频器输出的clk_i信号形成形成位同步信号
	wire bitSync_i,bitSync_q;
	syncout u7 (
   	 .rst   (rst),
		 .clk32 (clk),
	    .clk_i  (clk_i),
		 .Bit_Sync_i (bitSync_i),
		 .Bit_Sync_q  (bitSync_q));
	 assign Bit_Sync = bitSync_i;	
	 
	 //积分模块，根据位同步脉冲，在一个周期内对输入数据进行积分运算
	 integrated u8 (
   	 .rst   (rst),
		 .clk32 (clk),
		 .bitsync (bitSync_i),
		 .din  (datain),
		 .dout (int_i));

	 integrated u9 (
   	 .rst   (rst),
		 .clk32 (clk),
		 .bitsync (bitSync_q),
		 .din  (datain),
		 .dout (int_q));		 
		 
endmodule

	