module sync_top(
		rst			,
		clk			,
		datain		,	//输入数据
		dataout		,	//输出数据
		sin			,	//同步后输出载波分量
		df				,	//环路滤波器输出数据
		bit_sync			//位同步信号

);

	input						rst;   			//复位信号，高电平有效
	input						clk;   			//FPGA系统时钟：32MHz
	input	 signed [7:0]	datain;    		//输入数据：32MHz
	output signed [7:0]	sin;   			//同步后的载波输出信号(同相支路)
	output signed [27:0]	df;     			//环路滤波器输出数据
	output 					bit_sync;      //位同步脉冲输出
	output					dataout;
	
	wire signed [27:0]	di;
	wire signed [27:0]	df_reg;
	wire						bit_sync_reg;
	wire signed [7:0]		sine; 
	
	CostasLoop u1(
		.rst		(rst),
		.clk		(clk),
		.din		(datain),
		.sine		(sine),
		.di		(di),
		.df		(df_reg)
	); 
	
	BitSync u2(
		.rst			(rst),
		.clk			(clk),
		.datain		(di),
		.Bit_Sync	(bit_sync_reg)
	);
	
	assign 			df = df_reg;
	assign			bit_sync = bit_sync_reg;
	assign 			sin = sine;
	
	reg				dataout_reg;
	reg	[4:0]		cnt;
	always @(posedge clk or posedge rst)begin
		if (rst)
		   begin 
				dataout_reg <= 0;
				cnt <= 0;
			end
		else begin
			if (bit_sync_reg)
				cnt <= 0;
			else
				cnt <= cnt + 1'b1;
			if(cnt == 5'd16)
				dataout_reg <= ~di[27];
			else 
				dataout_reg <= dataout_reg;
		end
	end
	
	assign dataout = dataout_reg;
endmodule

